\name{venice}
\alias{venice}
\title{Largest Sea Levels in Venice}
\usage{venice}
\description{
  The \code{venice} data frame has 51 rows and 10 columns.
  The jth column contains the jth largest sea levels in Venice,
  for the years 1931--1981.
  Only the largest six measurements are available for the year 1935;
  the corresponding row contains four missing values.
  The years for each set of measurements are given as row names.
  A larger version of this data is available in the dataset 
  \code{venice2}.  
}
\format{A data frame with 51 rows and 10 columns.}
\source{
  Smith, R. L. (1986)
  Extreme value theory based on the \eqn{r} largest annual events.
  \emph{Journal of Hydrology}, \bold{86}, 27--43.
}
\references{
  Coles, S. G. (2001)
  \emph{An Introduction to Statistical Modeling of Extreme Values}.
  London: Springer-Verlag.
}
\keyword{datasets}
